body {
  padding: 0;
  background-color: #f9f9f9;
  font-family: sans-serif;
}
.container {
  margin: 0;
  display: flex;
  flex-direction: column;
  padding-bottom: 30px;
}

#header {
  padding: 10px 0px;
}
#header a {
  font-size: 20px;
  font-weight: 400;
  text-decoration: none;
}

#pane {
  display: flex;
  flex-direction: row;
}
#inputWrapper {
  flex: 1;
}
#input {
  margin-top: 3px;
}

#input .CodeMirror {
  border: 1px solid #ddd;
  height: auto;
}
#path {
  font-size: 20px;
  padding: 6px 8px;
  margin: 0;
  margin-bottom: 10px;
}
#output {
  flex: 0.3;
  padding: 20px;
  background: #f9f9f9;
}

h1 {
  margin: 10px;
}

#footer {
  position: fixed;
  text-align: right;
  bottom: 0px;
  left:0px;
  right: 0px;
  padding: 5px 20px;
  background: #eee;
  opacity: 0.8;
  z-index: 100;
  font-size: 13px;
  border-top: 1px solid #ddd;
}
